/*
 * SWE Ereigniskalender - Version 2
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax: +49-341-49067-15
 * E-Mail: info@bitctrl.de
 */

package de.bsvrz.vew.ereigniskal.data;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.vew.ereigniskal.module.EreignisGueltigkeit;
import de.bsvrz.vew.ereigniskal.module.EreignisZustandsWechsel;
import de.bsvrz.vew.syskal.SystemkalenderGueltigkeit;

/**
 * Repräsentation eines Ereignisses aus dem Datenverteiler-Modell.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public abstract class Ereignis {

    private ZeitlicheGueltigkeit zeitlicheGueltigkeit;
    private VerkehrlicheGueltigkeit verkehrlicheGueltigkeit;
    private RaeumlicheGueltigkeit raeumlicheGueltigkeit;
    private final String name;

    private SystemObject systemObject;
    private SystemObject ereignisTyp;

    /**
     * erzeugt ein neues Ereignis mit dem übergebenen Name
     * 
     * @param name
     *            der Name
     */
    public Ereignis(String name) {
        this.name = name;
    }

    /**
     * setzt die räumliche Gültigkeit des Ereignisses.
     * 
     * @param raeumlicheGueltigkeit
     *            die räumliche Gültigkeit
     */
    public void setRaeumlicheGueltigkeit(RaeumlicheGueltigkeit raeumlicheGueltigkeit) {
        this.raeumlicheGueltigkeit = raeumlicheGueltigkeit;
    }

    /**
     * setzt die zeitliche Gültigkeit des Ereignisses.
     * 
     * @param zeitlicheGueltigkeit
     *            die zeitliche Gültigkeit
     */
    public void setZeitlicheGueltigkeit(ZeitlicheGueltigkeit zeitlicheGueltigkeit) {
        this.zeitlicheGueltigkeit = zeitlicheGueltigkeit;
    }

    /**
     * setzt die verkehrliche Gültigkeit des Ereignisses.
     * 
     * @param verkehrlicheGueltigkeit
     *            die verkehrliche Gültigkeit
     */
    public void setVerkehrlicheGueltigkeit(VerkehrlicheGueltigkeit verkehrlicheGueltigkeit) {
        this.verkehrlicheGueltigkeit = verkehrlicheGueltigkeit;
    }

    /**
     * ermittelt die Ereignisgültigkeit des Ereignisses.
     * 
     * Die Ereignisgültigkeit ergibt sich aus der zeitlichen und verkehrlichen
     * Gültigkeit. Die Betrachtung der räumlichen Gültigkeit erfolgt
     * ausschließlich über die Anfrageschnittstelle.
     * 
     * @param zeitPunkt
     *            der Zeitpunkt dür den die Gültigkeit ermittelt werden soll.
     * @return die berechnete Gültigkeit
     */
    public EreignisGueltigkeit berechneEreignisGueltigkeit(LocalDateTime zeitPunkt) {

        SystemkalenderGueltigkeit zeitlich = SystemkalenderGueltigkeit.NICHT_GUELTIG;
        List<VerkehrlichesIntervall> verkehrlich = new ArrayList<>();

        if (zeitlicheGueltigkeit != null) {
            zeitlich = zeitlicheGueltigkeit.getGueltigkeit(zeitPunkt);
        }

        LocalDateTime wechselZeitpunkt = zeitlich.getNaechsterWechsel().getZeitPunkt();
        boolean zeitlichGueltig = zeitlich.isZeitlichGueltig();

        EreignisGueltigkeit ereignisGueltigkeit;
        if (verkehrlicheGueltigkeit == null || verkehrlicheGueltigkeit.isObsolete()) {
            ereignisGueltigkeit = EreignisGueltigkeit.of(zeitlich.isZeitlichGueltig(), zeitlich.isZeitlichGueltig(),
                    wechselZeitpunkt);
        } else {
            verkehrlich.addAll(verkehrlicheGueltigkeit.bestimmeRelevanteBereiche(zeitPunkt, zeitlicheGueltigkeit));
            ereignisGueltigkeit = EreignisGueltigkeit.of(zeitlich.isZeitlichGueltig(), false,
                    wechselZeitpunkt);
        }

        for (VerkehrlichesIntervall intervall : verkehrlich) {

            boolean verkehrlichGueltig = true;

            if (intervall.startsBefore(wechselZeitpunkt) && intervall.endsAfter(zeitPunkt)) {
                if (intervall.startsAfter(zeitPunkt)) {
                    verkehrlichGueltig = false;
                    wechselZeitpunkt = intervall.getStart();
                } else if (intervall.endsBefore(wechselZeitpunkt)) {
                    wechselZeitpunkt = intervall.getEnde();
                }
                ereignisGueltigkeit = EreignisGueltigkeit.of(zeitlichGueltig, verkehrlichGueltig,
                        wechselZeitpunkt);
            }
        }

        return ereignisGueltigkeit;
    }

    /**
     * liefert die Liste der Gültigkeitszustandswechsel für den übergebenen
     * Zeitbereich.
     * 
     * Ein Zustandswechsel erfolgt, wenn sich die zeitliche und/oder die
     * verkehrliche Gültigkeit ändert. Geliefert werden alle Wechsel, die sich
     * innerhalb des angefragten Zeitraums vollziehen und der Wechsel auf einen
     * gültigen Zustand, der zum Startzeitpunkt des Anfragezeitraums besteht.
     * 
     * @param anfangsZeitpunkt
     *            der Anfangszeitpunkt des Anfragezeitraums
     * @param endZeitpunkt
     *            der Endzeitpunkt des Anfragezeitraums
     * @return die Liste der Zustandswechsel
     */
    public List<EreignisZustandsWechsel> getEreignisZustandsWechsel(LocalDateTime anfangsZeitpunkt,
            LocalDateTime endZeitpunkt) {

        List<EreignisZustandsWechsel> result = new ArrayList<>();
        LocalDateTime aktuellerZeitPunkt = anfangsZeitpunkt;

        do {
            EreignisGueltigkeit ereignisGueltigkeit = berechneEreignisGueltigkeit(aktuellerZeitPunkt);
            if (!aktuellerZeitPunkt.isAfter(endZeitpunkt)) {
                if (!result.isEmpty() || ereignisGueltigkeit.isGueltig()) {
                    result.add(EreignisZustandsWechsel.of(aktuellerZeitPunkt, this,
                            ereignisGueltigkeit.isZeitlichGueltig(), ereignisGueltigkeit.isVerkehrlichGueltig()));
                }
                aktuellerZeitPunkt = ereignisGueltigkeit.getWechselZeit();
            }
        } while (!aktuellerZeitPunkt.isAfter(endZeitpunkt));

        return result;
    }

    /**
     * liefert die zuegordnete räumliche Gültigkeit.
     * 
     * @return die räumliche Gültigkeit
     */
    public RaeumlicheGueltigkeit getRaeumlicheGueltigkeit() {
        if (raeumlicheGueltigkeit == null) {
            raeumlicheGueltigkeit = new RaeumlicheGueltigkeit();
        }
        return raeumlicheGueltigkeit;
    }

    /**
     * liefert den Name des Ereignisses.
     * 
     * @return den Name
     */
    public String getName() {
        return name;
    }

    /**
     * liefert das zugeordnete Systemobjekt des Ereignisses.
     * 
     * @return das Systemobjekt
     */
    public SystemObject getSystemObject() {
        return systemObject;
    }

    /**
     * setzt das zugeordnete Systemobjekt des Ereignisses.
     * 
     * @param systemObject
     *            das Systemobjekt
     */
    protected void setSystemObject(SystemObject systemObject) {
        this.systemObject = systemObject;
    }

    /**
     * setzt den Ereignistyp des Ereignisses.
     * 
     * Der Ereignistyp ist nur für Filterungen bei Anfragen über die
     * Anfrageschnittstelle von Bedeutung, der Ereignistyp hat keine Bedeutung
     * für die Gültigkeit von Ereignissen.
     * 
     * @param ereignisTyp
     *            der Typ
     */
    protected void setEreignisTyp(SystemObject ereignisTyp) {
        this.ereignisTyp = ereignisTyp;
    }

    /**
     * liefert den Ereignistyp des Ereignisses.
     * 
     * @return den Typ
     */
    public SystemObject getEreignisTyp() {
        return ereignisTyp;
    }
}
