/*
 * SWE Ereigniskalender - Version 2
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax: +49-341-49067-15
 * E-Mail: info@bitctrl.de
 */

package de.bsvrz.vew.ereigniskal.module;

import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;

/**
 * Repräsentation der Gültigkeit eines Ereignisses.
 * 
 * Die Gültigkeit enthält Angaben zur zeitlichen und verkehrlichen Gültigkeit
 * eines Ereignisses und den Zeitpunkt des erwarteten nächsten Zustandswechsels.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class EreignisGueltigkeit {

    private boolean zeitlichGueltig;
    private boolean verkehrlichGueltig;
    private LocalDateTime wechselZeit = LocalDateTime.now();

    /**
     * liefert, ob zeitliche Gültigkeit besteht.
     * 
     * @return die Gültigkeit
     */
    public boolean isZeitlichGueltig() {
        return zeitlichGueltig;
    }

    /**
     * liefert, ob verkehrliche Gültigkeit besteht.
     * 
     * @return die Gültigkeit
     */
    public boolean isVerkehrlichGueltig() {
        return verkehrlichGueltig;
    }

    /**
     * liefert den voraussichtlich nächsten Zustandswechsel als Millisekunden
     * der lokalen Zeit nach dem 1.1.1970.
     * 
     * @return den Zeitpunkt
     */
    public long getVoraussichtlicheAenderung() {
        return ZonedDateTime.of(wechselZeit, ZoneId.systemDefault()).toInstant().toEpochMilli();
    }

    /**
     * erzeugt eine neue Instanz der {@link EreignisGueltigkeit} mit den
     * übergebenen Daten
     * 
     * @param zeitlich
     *            die zeitliche Gültigkeit
     * @param verkehrlich
     *            die verkehrliche Gültigkeit
     * @param wechsel
     *            der Zeitpunkt des voraussichtlich nächsten Zustandswechsels
     * @return die neue Instanz
     */
    public static EreignisGueltigkeit of(boolean zeitlich, boolean verkehrlich,
            LocalDateTime wechsel) {

        EreignisGueltigkeit gueltigkeit = new EreignisGueltigkeit();
        gueltigkeit.zeitlichGueltig = zeitlich;
        gueltigkeit.verkehrlichGueltig = verkehrlich;
        gueltigkeit.wechselZeit = wechsel;

        return gueltigkeit;
    }

    /**
     * liefert den voraussichtlichen Zeitpunkt des nächsten Zustandswechsels.
     * 
     * @return den Zeitpunkt
     */
    public LocalDateTime getWechselZeit() {
        return wechselZeit;
    }

    @Override
    public String toString() {
        return "Z: " + zeitlichGueltig + ", V: " + verkehrlichGueltig + " --> " + wechselZeit;
    }

    /**
     * liefert die Gültigkeit (zeitlich oder verkehrlich gültig)
     * 
     * @return die Gültigkeit
     */
    public boolean isGueltig() {
        return isZeitlichGueltig() || isVerkehrlichGueltig();
    }
}
