/*
 * SWE Ereigniskalender - Version 2
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax: +49-341-49067-15
 * E-Mail: info@bitctrl.de
 */

package de.bsvrz.vew.ereigniskal.module;

import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;

import de.bsvrz.vew.ereigniskal.data.Ereignis;

/**
 * Definition des Zustandswechsels für ein Ereignis.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
/**
 * @author Peuker
 *
 */
public class EreignisZustandsWechsel {

    private LocalDateTime zeitpunkt;
    private Ereignis ereignis;
    private boolean zeitlichGueltig;
    private boolean verkehrlichGueltig;

    /**
     * erzeugt eine Instanz eines {@link EreignisZustandsWechsel} aus den
     * übergebenen Daten.
     * 
     * @param zeit
     *            der Zeitpunkt des Zustandswechsels
     * @param referenz
     *            das Ereignis, dessen Zustand geändert wird
     * @param zeitlich
     *            die zeitliche Gültigkeit nach dem Wechsel
     * @param verkehrlich
     *            die verkehrliche Gültigkeit nach dem Wechsel
     * @return die neue Instanz
     */
    public static EreignisZustandsWechsel of(LocalDateTime zeit, Ereignis referenz, boolean zeitlich,
            boolean verkehrlich) {

        EreignisZustandsWechsel wechsel = new EreignisZustandsWechsel();

        wechsel.zeitpunkt = zeit;
        wechsel.ereignis = referenz;
        wechsel.zeitlichGueltig = zeitlich;
        wechsel.verkehrlichGueltig = verkehrlich;

        return wechsel;
    }

    /**
     * liefert den Zeitpunkt des Zustandswechsels in Millisekunden seit
     * 1.1.1970.
     * 
     * @return den Zeitpunkt
     */
    public long getZeitpunktMillis() {
        return ZonedDateTime.of(zeitpunkt, ZoneId.systemDefault()).toInstant().toEpochMilli();
    }

    /**
     * liefert das Ereignis, dessen Zustand sich ändert.
     * 
     * @return das Ereignis
     */
    public Ereignis getEreignis() {
        return ereignis;
    }

    /**
     * liefert die zeitliche Gültigkeit
     * 
     * @return die Gültigkeit
     */
    public boolean getZeitlichGueltig() {
        return zeitlichGueltig;
    }

    /**
     * liefert die verkehrlichee Gültigkeit
     * 
     * @return die Gültigkeit
     */
    public boolean getVerkehrlichGueltig() {
        return verkehrlichGueltig;
    }

    /**
     * liefert den Zeitpunkt des Zustandswechsels.
     * 
     * @return den Zeitpunkt
     */
    public LocalDateTime getZeitPunkt() {
        return zeitpunkt;
    }

    @Override
    public String toString() {
        return "EreignisZustandsWechsel [zeitpunkt=" + zeitpunkt + ", ereignis=" + ereignis.getName()
                + ", zeitlichGueltig="
                + zeitlichGueltig + ", verkehrlichGueltig=" + verkehrlichGueltig + "]";
    }

}
